import axios from 'axios';
import Log from './Log.js';
import LogFmt from './LogFmt.js';

export default class Request {

  url;
  method;
  data;
  headers;

  response;
  statusCode;
  responseata;
  responseHeaders;

  constructor(url, method, data = null, headers = null) {
    this.url = url;
    this.method = method;
    this.data = data;
    this.headers = headers;
  }

  async _request() {
    try {
      const config = this._generateRequestConfig();

      this.response = await axios(config);
      this.responseData = this.response.data;
      this.statusCode = this.response.status;
      this.responseHeaders = this.response.headers;

      Log.debug(
        LogFmt.name('request')
          .string('method', this.method)
          .number('status', this.statusCode)
          .string('url', this.url)
          .log()
      );
      // FIXME: Pretty print. Log.debug(`Headers: ${JSON.stringify(this.responseHeaders)}`);
    } catch (error) {
      this.response = this._findError(error);
      this.responseHeaders = this.response.headers;
      this.statusCode = this.response.status;

      if (!this._shouldIgnore()) {
        this._processError();
      }
    }
    return this;
  }

  _generateRequestConfig() {
    let config = {
      method: this.method,
      url: this.url,
      headers: this._addNeccesaryHeaders()
    };
    if (this.data) {
      config.data = JSON.stringify(this.data);
    }
    return config;
  }

  _addNeccesaryHeaders() {
    return this.headers;
  }

  _findError(error) {
    if (error.response) {
      return error.response;
    } else if (error.request) {
      return error.request;
    }
    return error;
  }

  _processError() {
    const logFmt = LogFmt.name('request');
    if (this.response.status) {
      Log.error(
        logFmt
          .string('method', this.method)
          .number('status', this.statusCode)
          .string('url', this.url)
          .log()
      );
      // FIXME: Pretty print.
      // Log.error(`Data: ${JSON.stringify(this.response.data)}`);
      // Log.error(`Headers: ${JSON.stringify(this.response.headers)}`);
    } else {
      Log.error(
        logFmt
          .string('parsed-error', this._processStringError())
          .log()
      );
    }
  }

  _processStringError() {
    let stringError;
    try {
      stringError = JSON.stringify(this.response);
    } catch (error) {
      stringError = this.response;
    }
    return stringError;
  }

  _shouldIgnore() {
    return false;
  }

}
